<?php

require_once NIC_BASE . '/lib/Device/SNMP.php';

/**
 * Nic_Device_www:: Nic_Device implementation to retrieve data from a
 * Squid Proxy server via SNMP.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @since   Nic 0.2
 * @package Nic
 */
class Nic_Device_squid extends Nic_Device_SNMP {

    /* SNMP Object ID's to retrieve */
    var $oids = array(
        array('name' => 'Uptime', 'oid' => '.1.3.6.1.4.1.3495.1.1.3.0', 'storeable' => false),
        array('name' => 'Version', 'oid' => '.1.3.6.1.4.1.3495.1.2.3.0', 'storeable' => false),
        array('name' => 'Number of clients accessing cache', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.15.0', 'storeable' => true),
        array('name' => 'Number of HTTP requests received', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.1.0', 'storeable' => true),
        array('name' => 'Number of HTTP Hits', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.2.0', 'storeable' => true),
        array('name' => 'Number of HTTP Errors', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.3.0', 'storeable' =>true),
        array('name' => 'Number of HTTP KB\'s recieved', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.4.0', 'storeable' => true),
        array('name' => 'Number of HTTP KB\'s transmitted', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.5.0', 'storeable' => true),
        array('name' => 'Number of ICP messages sent', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.6.0', 'storeable' => true),
        array('name' => 'Number of ICP messages received', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.7.0', 'storeable' => true),
        array('name' => 'KB\'s of traffic recieved from servers', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.12.0', 'storeable' => true),
        array('name' => 'KB\'s of traffic sent to servers', 'oid' => '.1.3.6.1.4.1.3495.1.3.2.1.13.0', 'storeable' => true),
        array('name' => 'Amount of cpu seconds consumed', 'oid' => '.1.3.6.1.4.1.3495.1.3.1.4.0', 'storeable' => true)
        );

    /* Array containing the details of graphs available for this device. */
    var $_graphs = array(
        array(
            'title' => 'Squid Proxy Server Usage',
            'axisLabel' => 'Number of Cleints',
            'data' => array(
                array(
                    'source' => 'Number of clients accessing cache',
                    'legend' => 'Number of Cleints Accessing Cache',
                    'description' => 'The number of different IP addresses using this cache since it started',
                    'type' => NIC_DATA_INTEGER
                )
            )
        ),
        array(
            'title' => 'Squid Proxy Server Usage',
            'axisLabel' => 'Percentage',
            'data' => array(
                array(
                    'source' => 'Amount of cpu seconds consumed',
                    'legend' => 'CPU Usage',
                    'description' => 'The percentage of time the CPU is actually active',
                    'type' => NIC_DATA_COUNTER,
                    'scale_function' => '*',
                    'scale_factor' => 100,
                )
            )
        ),
        array(
            'title' => 'HTTP Requets',
            'axisLabel' => 'Requests / Second',
            'data' => array(
                array(
                    'source' => 'Number of HTTP requests received',
                    'legend' => 'Total HTTP Requests',
                    'description' => 'The total number of HTTP requests recieved per second.',
                    'type' => NIC_DATA_COUNTER
                ),
                array(
                    'source' => 'Number of HTTP Hits',
                    'legend' => 'HTTP Hits',
                    'description' => 'Number of requests the were fullfilled from the cache.',
                    'type' => NIC_DATA_COUNTER
                ),
                array(
                    'source' => 'Number of HTTP Errors',
                    'legend' => 'HTTP Errors',
                    'description' => 'Number of requests the were results in HTTP errors.',
                    'type' => NIC_DATA_COUNTER
                )
            )
        ),
        array(
            'title' => 'HTTP Client Tranfers',
            'axisLabel' => 'KB / Second',
            'data' => array(
                array(
                    'source' => 'Number of HTTP KB\'s transmitted',
                    'legend' => 'KBs Sent',
                    'description' => 'The amount of data per second sent to clients of this cache.',
                    'type' => NIC_DATA_COUNTER
                ),
                array(
                    'source' => 'Number of HTTP KB\'s recieved',
                    'legend' => 'KBs Recieved',
                    'description' => 'The amount of data per second recieved from clients of this cache.',
                    'type' => NIC_DATA_COUNTER
                )
            )
        ),
        array(
            'title' => 'Server Tranfers',
            'axisLabel' => 'KB / Second',
            'data' => array(
                array(
                    'source' => 'KB\'s of traffic recieved from servers',
                    'legend' => 'KBs Recieved',
                    'description' => 'The amount of data per second recieved by this cache from servers.',
                    'type' => NIC_DATA_COUNTER
                ),
                array(
                    'source' => 'KB\'s of traffic sent to servers',
                    'legend' => 'KBs Sent',
                    'description' => 'The amount of data per second sent by this cache to servers.',
                    'type' => NIC_DATA_COUNTER
                )
            )
        ),
        array(
            'title' => 'ICP Requests',
            'axisLabel' => 'Requests / Second',
            'data' => array(
                array(
                    'source' => 'Number of ICP messages sent',
                    'legend' => 'Requests Sent',
                    'description' => 'The number of ICP (Internet Cache Protocaol) Messages sent to other caches.',
                    'type' => NIC_DATA_COUNTER
                ),
                array(
                    'source' => 'Number of ICP messages received',
                    'legend' => 'Requests Recieved',
                    'description' => 'The number of ICP (Internet Cache Protocaol) Messages recieved from other caches.',
                    'type' => NIC_DATA_COUNTER
                )
            )
        )
    );

    function Nic_Device_squid($params = array())
    {
        $this->params = $params;
        if (!isset($this->params['timeout'])) {
            $this->params['timeout'] = 10;
        }
        if (!isset($this->params['squidport'])) {
            $this->params['squidport'] = 3128;
        }
    }

    function getStatus()
    {
        $status = array();
        $status['status'] = NIC_STATUS_FAULT;
        $status['message'] = _("Unable to proxy server");

        require_once 'Net/Socket.php';
        $socket = &new Net_Socket();
        if (!@$socket->connect($this->params['hostspec'], $this->params['squidport'], false, 3)) {
            return $status;
        } else {
            $socket->write("GET cache_object://" . $this->params['hostspec'] . "/info HTTP/1.0\r\n\r\n");

            // Skip to end of headers.
            $last = '';
            do {
                $last = $socket->readLine();
            } while (!empty($last) && !$socket->eof());

            // Get the info
            $result = '';
            do {
                $last = $socket->readLine();
                $result .= $last . "\n";
            } while (!$socket->eof());

            $socket->disconnect();

            $status['status'] = NIC_STATUS_NORMAL;
            $status['message'] = $result;

            return $status;
        }

        /*
        $uptime = @snmpget ($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], '.1.3.6.1.4.1.3495.1.1.3.0', $this->params['timeout'], 5);
        $status = array();
        $status['status'] = ($uptime === false) ? NIC_STATUS_FAULT : NIC_STATUS_NORMAL;
        $status['message'] = ($uptime === false) ? 'Unable to retrieve uptime': sprintf(_("Uptime: %s"), $this->_cleanResponse($uptime));
        */
        return $status;
    }

}
